package mailinglist.client;

import java.io.IOException;
import java.net.InetSocketAddress;
import java.nio.ByteBuffer;
import java.util.UUID;
import java.util.concurrent.CountDownLatch;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.SimpleFormatter;

import com.sun.grizzly.BaseSelectionKeyHandler;
import com.sun.grizzly.Controller;
import com.sun.grizzly.ControllerStateListenerAdapter;
import com.sun.grizzly.DefaultProtocolChain;
import com.sun.grizzly.ProtocolChain;
import com.sun.grizzly.ProtocolChainInstanceHandler;
import com.sun.grizzly.SelectorHandler;
import com.sun.grizzly.TCPConnectorHandler;
import com.sun.grizzly.TCPSelectorHandler;

/**
 * @author kloh
 * 
 */
public class Client {

	// How many main-blocks should there be?
	private final static int sMAIN_COUNTER = 5;
	// How many runners for each block?
	private final static int sRUNNER_COUNTER = 5;

	private Controller mReceiveFileGrizzlyController;
	private TCPConnectorHandler mReceiveFileConnectorHandler;

	private final CountDownLatch mCOUNT_DOWN_LATCH = new CountDownLatch(1);

	private final static CountDownLatch[] sSHARED_COUNT_DOWN_LATCH = new CountDownLatch[sMAIN_COUNTER];

	static {
		// just for System.outs
		Controller.logger().setUseParentHandlers(false);
		Controller.logger().setLevel(Level.WARNING);
		Controller.logger().addHandler(new Handler() {

			final Formatter mFORMATTER = new SimpleFormatter();

			@Override
			public void publish(final LogRecord pRecord) {
				if (pRecord.getLevel().intValue() < Level.WARNING.intValue())
					System.out.print(this.mFORMATTER.format(pRecord));
				else
					System.err.print(this.mFORMATTER.format(pRecord));
			}

			@Override
			public void flush() {
				// TODO Auto-generated method stub

			}

			@Override
			public void close() throws SecurityException {
				// TODO Auto-generated method stub

			}
		});
	}

	public static void main(String[] args) {

		for (int i = 0; i < sMAIN_COUNTER; i++) {
			// create for each main-block a countdownlatch...
			sSHARED_COUNT_DOWN_LATCH[i] = new CountDownLatch(sRUNNER_COUNTER);
			// ...and start sRUNNER_COUNTER threads
			for (int j = 0; j < sRUNNER_COUNTER; j++)
				new Thread(getClientStarter(i, j)).start();
		}

	}

	private static Runnable getClientStarter(final int pMainCounter,
			final int pRunnerCounter) {

		return new Runnable() {

			public void run() {

				Client lClient = new Client();

				lClient.start_ReceiveFileServer_GrizzlyController();

				// get a message buffer
				final ByteBuffer lBUFFER = lClient.getProtocolBuffer(
						pMainCounter, pRunnerCounter);

				// just for synchronization
				sSHARED_COUNT_DOWN_LATCH[pMainCounter].countDown();
				try {
					sSHARED_COUNT_DOWN_LATCH[pMainCounter].await();
				} catch (InterruptedException e1) {
					e1.printStackTrace();
				}

				try {
					lClient.mReceiveFileConnectorHandler.write(lBUFFER, true);
				} catch (IOException e) {
					e.printStackTrace();
				}

				System.out.println(pMainCounter + " | " + pRunnerCounter
						+ ": Written to server");

				// wait for signal from ClientForwardFilter
				lClient.waitForCountDown();

				lClient.stop_ReceiveFile_GrizzlyController();

			}
		};

	}

	private void start_ReceiveFileServer_GrizzlyController() {

		final CountDownLatch lCountDownLatch = new CountDownLatch(1);
		this.mReceiveFileGrizzlyController = new Controller();
		final SelectorHandler lSelectorHandler = new TCPSelectorHandler(true);

		lSelectorHandler.setSelectionKeyHandler(new BaseSelectionKeyHandler());
		this.mReceiveFileGrizzlyController.addSelectorHandler(lSelectorHandler);

		final DefaultProtocolChain lProtChain = new DefaultProtocolChain();
		lProtChain.addFilter(new ClientProtocolFilter());
		lProtChain.addFilter(new ClientForwardFilter(this.mCOUNT_DOWN_LATCH));

		this.mReceiveFileGrizzlyController
				.setProtocolChainInstanceHandler(new ProtocolChainInstanceHandler() {

					public ProtocolChain poll() {
						return lProtChain;
					}

					public boolean offer(final ProtocolChain instance) {
						return false;
					}
				});

		this.mReceiveFileGrizzlyController
				.addStateListener(new ControllerStateListenerAdapter() {

					@Override
					public void onException(final Throwable e) {
						throw new RuntimeException(e);
					}

					@Override
					public void onReady() {
						lCountDownLatch.countDown();
					}

				});

		new Thread(this.mReceiveFileGrizzlyController).start();
		boolean lRepeat = false;
		do {
			lRepeat = false;
			try {
				lCountDownLatch.await();
			} catch (final InterruptedException e) {
				lRepeat = true;
			}
		} while (lRepeat);

		// Der ReceiveFile-Grizzly-ClientConnector
		this.mReceiveFileConnectorHandler = (TCPConnectorHandler) this.mReceiveFileGrizzlyController
				.acquireConnectorHandler(Controller.Protocol.TCP);

		try {
			this.mReceiveFileConnectorHandler.connect(new InetSocketAddress(
					"localhost", 4444));
			if (!(this.mReceiveFileConnectorHandler).isConnected()) {
				this.stop_ReceiveFile_GrizzlyController();
			}
		} catch (final Throwable t) {
			this.stop_ReceiveFile_GrizzlyController();
			throw new RuntimeException(t);
		}

	}

	private void stop_ReceiveFile_GrizzlyController() {

		if (this.mReceiveFileConnectorHandler != null)
			try {
				this.mReceiveFileConnectorHandler.close();
			} catch (final IOException e) {
				throw new RuntimeException(e);
			}
		if (this.mReceiveFileGrizzlyController.isStarted())
			try {
				this.mReceiveFileGrizzlyController.stop();
			} catch (final IOException e) {
				throw new RuntimeException(e);
			}
	}

	private void waitForCountDown() {
		try {
			this.mCOUNT_DOWN_LATCH.await();
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}

	private ByteBuffer getProtocolBuffer(int pMainCounter, int pRunnerCounter) {

		final String lMESSAGE = pMainCounter
				+ " | "
				+ pRunnerCounter
				+ ": Lorem ipsum et zzril tantas similique per, has facer fuisset appellantur eu. Ne vidisse copiosae necessitatibus eam, principes assentior complectitur vix ne. Dicat quaeque impedit an mel, te pri iudico dissentias adversarium. Ad accusam albucius vim, ei homero qualisque evertitur mei. An ornatus dolorem imperdiet vix, ei eam diam voluptatibus. Lorem vivendo volumus mel in, probo minimum propriae ut sit. Tantas repudiare cotidieque sed eu, te est dicunt prompta periculis.";

		final ByteBuffer lBUFFER = ByteBuffer.allocateDirect(36 + 36 + 36 + 4
				+ lMESSAGE.length());

		lBUFFER.put(UUID.randomUUID().toString().getBytes());

		lBUFFER.put(UUID.randomUUID().toString().getBytes());

		lBUFFER.put(UUID.randomUUID().toString().getBytes());

		lBUFFER.putInt(lMESSAGE.getBytes().length);
		lBUFFER.put(lMESSAGE.getBytes());

		lBUFFER.flip();

		return lBUFFER;

	}

}
