package mailinglist.client;

import java.nio.ByteBuffer;

import com.sun.grizzly.ProtocolParser;
import com.sun.grizzly.util.WorkerThread;

/**
 * @author kloh
 * 
 */
public class ClientProtocolParser implements
		ProtocolParser<String> {

	private ByteBuffer mSavedBuffer;
	private int mSavedPosition;

	private boolean mWantsMoreData;
	private boolean mHasUnparsedData;

	private String mMessage;

	public final static int sDEFAULT_BYTEBUFFER_SIZE = 8192;

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.grizzly.ProtocolParser#getNextMessage()
	 */
	public String getNextMessage() {

		if (this.mMessage == null)
			this.mMessage = extractMessage();

		final String lTmp = new String(this.mMessage);

		this.mMessage = null;

		return lTmp;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.grizzly.ProtocolParser#hasMoreBytesToParse()
	 */
	public boolean hasMoreBytesToParse() {
		return this.mHasUnparsedData;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.grizzly.ProtocolParser#hasNextMessage()
	 */
	public boolean hasNextMessage() {

		if (this.mMessage == null)
			this.mMessage = extractMessage();

		return this.mMessage != null;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.grizzly.ProtocolParser#isExpectingMoreData()
	 */
	public boolean isExpectingMoreData() {
		return this.mWantsMoreData;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.grizzly.ProtocolParser#releaseBuffer()
	 */
	public boolean releaseBuffer() {

		if (this.mWantsMoreData) {
			reallocateBuffer();
		} else {
			this.mSavedBuffer = ByteBuffer.allocate(sDEFAULT_BYTEBUFFER_SIZE);
		}

		return this.mWantsMoreData;

	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see com.sun.grizzly.ProtocolParser#startBuffer(java.nio.ByteBuffer)
	 */
	public void startBuffer(final ByteBuffer pByteBuffer) {
		this.mSavedBuffer = pByteBuffer;
		this.mSavedBuffer.flip();
	}

	private String extractMessage() {

		if (!this.mSavedBuffer.hasRemaining()) {
			this.mWantsMoreData = false;
			this.mHasUnparsedData = false;
			this.mSavedBuffer.clear();
			return null;
		}

		this.mSavedPosition = this.mSavedBuffer.position();

		if (this.mSavedBuffer.remaining() < 4) {
			// <=> Nachricht unvollstndig
			if (this.mSavedBuffer.position() != this.mSavedPosition)
				this.mSavedBuffer.position(this.mSavedPosition);
			this.mWantsMoreData = true;
			this.mHasUnparsedData = false;
			return null;
		}
		final int lMESSAGE_SIZE = this.mSavedBuffer.getInt();
		
		if (this.mSavedBuffer.remaining() < lMESSAGE_SIZE) {
			// <=> Nachricht unvollstndig
			if (this.mSavedBuffer.position() != this.mSavedPosition)
				this.mSavedBuffer.position(this.mSavedPosition);
			this.mWantsMoreData = true;
			this.mHasUnparsedData = false;
			return null;
		}

		final byte[] lContent = new byte[lMESSAGE_SIZE];
		this.mSavedBuffer.get(lContent);
		final String lMessage = new String(lContent);
		
		if (this.mSavedBuffer.remaining() == 0) {
			this.mHasUnparsedData = false;
			this.mWantsMoreData = false;
		} else {
			this.mHasUnparsedData = true;
			this.mWantsMoreData = false;
			this.mSavedPosition = this.mSavedBuffer.position();
		}

		return lMessage; 

	}

	/**
	 * Diese Methode erstellt einen neuen ByteBuffer, der die Bytes im alten
	 * Buffer erhlt und Platz fr weitere hat
	 * 
	 * @param pBufferSize
	 */
	private void reallocateBuffer() {

		// Hat der aktuelle Buffer keinen freien Platz mehr, wird die doppelte
		// Standard-Buffer-Gre genommen
		int lNewBufferSize = this.mSavedBuffer.remaining() == 0 ? sDEFAULT_BYTEBUFFER_SIZE * 2
				: sDEFAULT_BYTEBUFFER_SIZE;

		// Solange die neue Buffer-Gre nicht fr die im aktuellen Buffer
		// befindlichen Bytes ausreicht...
		while (lNewBufferSize <= this.mSavedBuffer.remaining())
			// ...wird die neue Buffer-Gre erhht
			lNewBufferSize += sDEFAULT_BYTEBUFFER_SIZE;

		// Jetzt reicht die neue Buffer-Gre aus, um...
		final ByteBuffer lTmpBuffer = ByteBuffer.allocate(lNewBufferSize);
		// ... die im alten Buffer befindlichen Bytes zu bekommen (und
		// gleichzeitig noch Platz zu haben fr weitere Bytes)
		lTmpBuffer.put(this.mSavedBuffer);

		this.mSavedBuffer = lTmpBuffer;

		((WorkerThread) Thread.currentThread()).setByteBuffer(this.mSavedBuffer);

	}

}
