package mailinglist.server;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import com.sun.grizzly.Controller;
import com.sun.grizzly.DefaultProtocolChain;
import com.sun.grizzly.DefaultProtocolChainInstanceHandler;
import com.sun.grizzly.ProtocolChain;
import com.sun.grizzly.TCPSelectorHandler;

/**
 * @author kloh
 * 
 */
public class Server {

	private Controller mController;

	private TCPSelectorHandler mSelectorHandler = null;

	private final DefaultProtocolChain mPROTOCOL_CHAIN;

	private Thread mServerThread;

	public Server() {

		this.mPROTOCOL_CHAIN = new DefaultProtocolChain();

		this.mPROTOCOL_CHAIN.addFilter(new ServerProtocolFilter());
		this.mPROTOCOL_CHAIN.addFilter(new ServerForwardFilter());

	}

	private void configureServiceDispatcherHandler(
			final int pServiceFileDownloadPort) {

		if (this.mSelectorHandler == null) {

			this.mSelectorHandler = new TCPSelectorHandler();

			this.mSelectorHandler.setPort(pServiceFileDownloadPort);

			this.mSelectorHandler
					.setProtocolChainInstanceHandler(new DefaultProtocolChainInstanceHandler() {
						@Override
						public ProtocolChain poll() {
							return Server.this.mPROTOCOL_CHAIN;
						}
					});

		}

	}

	private void configureController(final int pPort) {

		if (this.mController == null) {

			this.mController = new Controller();

			configureServiceDispatcherHandler(pPort);
			this.mController.addSelectorHandler(this.mSelectorHandler);

		}

	}

	public void start_GrizzlyConnector() {

		final int lServiceFileDownloadPort = 4444;

		configureController(lServiceFileDownloadPort);

		this.mServerThread = new Thread(new Runnable() {

			public void run() {
				System.out.println("Listening @ port "
						+ lServiceFileDownloadPort + ".");

				try {
					Server.this.mController.start();
				} catch (final IOException e) {
					throw new RuntimeException(e);
				}

				System.out.println("Stopped listening @ port "
						+ lServiceFileDownloadPort + ".");
			}

		});
		this.mServerThread.setName(Server.class.getName());
		this.mServerThread.start();

	}

	public void stop_GrizzlyConnector() {

		if (this.mController != null) {
			try {
				this.mController.stop();
			} catch (final IOException e) {
				throw new RuntimeException(e);
			} finally {
				if (this.mServerThread != null)
					this.mServerThread.interrupt();
				this.mServerThread = null;
				this.mSelectorHandler = null;
				this.mController = null;
			}
		}

	}

	public static void main(String[] args) {

		Server lServer = new Server();

		lServer.start_GrizzlyConnector();

		System.out.println("Press [q] + [enter] to stop Server.");
		try {
			BufferedReader lBR = new BufferedReader(new InputStreamReader(
					System.in));
			String lKey;
			while ((lKey = lBR.readLine()) != null)
				if (lKey.equals("q"))
					break;
		} catch (IOException e) {
			e.printStackTrace();
		}

		lServer.stop_GrizzlyConnector();

	}

}
